@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/venus/up.c,v 1.17.2.3 2007/11/26 21:08:45 shadow Exp $");
 
 /* missing type from C language */
 
@@ -44,18 +42,17 @@ RCSID
 #endif
 
 #include <string.h>
+#include <stdlib.h>
 
 #include <afs/venus.h>
+#include <afs/sys_prototypes.h>
+#include <afs/afsutil.h>
+#include <afs/afs_consts.h>
 
 /* ************************************************************* */
 
 #define MAXACL 400
 
-#if !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV) && !defined(AFS_XBSD_ENV)
-extern sys_nerr;
-extern char *sys_errlist[];
-#endif
-
 short verbose = 0;
 short renameTargets = 0;
 short oneLevel = 0;
@@ -63,13 +60,11 @@ short preserveDate = 1;
 short preserveMountPoints = 0;
 short forceOverwrite = 0;
 
-int pageSize;
 short setacl = 1;
 short oldAcl = 0;
 char file1[MAXPATHLEN], file2[MAXPATHLEN];
 
-#define	MAXSIZE	2048
-static char space[MAXSIZE];
+static char space[AFS_PIOCTL_MAXSIZE];
 
 struct OldAcl {
     int nplus;
@@ -97,8 +92,8 @@ main(int argc, char *argv[])
 {
 #ifdef	AFS_AIX32_ENV
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -108,9 +103,6 @@ main(int argc, char *argv[])
     nsa.sa_handler = SIG_DFL;
     nsa.sa_flags = SA_FULLDUMP;
     sigaction(SIGSEGV, &nsa, NULL);
-#endif
-#if !defined (AFS_AIX_ENV) && !defined (AFS_HPUX_ENV)
-    pageSize = getpagesize();
 #endif
     ScanArgs(argc, argv);
 
@@ -133,6 +125,10 @@ ScanArgs(int argc, char *argv[])
     while (argc > 0 && *argv[0] == '-') {
 	char *cp = *argv;
 
+	if (strlen(cp) > 2) {
+	    goto badoption;
+	}
+
 	switch (*++cp) {
 	case 'v':
 	    verbose = 1;
@@ -159,7 +155,10 @@ ScanArgs(int argc, char *argv[])
 	    break;
 
 	default:
-	    fprintf(stderr, "Unknown option: '%c'\n", *cp);
+	    cp--;
+
+ badoption:
+	    fprintf(stderr, "Unknown option: '%s'\n", cp);
 	    fprintf(stderr, USAGE);
 	    exit(1);
 	}
@@ -191,11 +190,11 @@ MakeParent(char *file, afs_int32 owner)
     char *p;
     struct stat s;
 
-    strcpy(parent, file);
+    strlcpy(parent, file, sizeof parent);
 
     p = strrchr(parent, '/');
     if (!p) {
-	strcpy(parent, ".");
+	strlcpy(parent, ".", sizeof parent);
     } else if (p > parent) {
 	*p = '\0';
     } else {
@@ -278,32 +277,26 @@ Copy(char *file1, char *file2, short recursive, int level)
 
 	    if (verbose) {
 		printf("  Copy file %s to %s (%u Bytes)\n", file1, file2,
-		       s1.st_size);
+		       (unsigned int) s1.st_size);
 		fflush(stdout);
 	    }
 
-	    strcpy(tmpfile, file2);	/* Name of temporary file */
-	    strcat(tmpfile, ".UPD");
+	    strlcpy(tmpfile, file2, sizeof tmpfile);	/* Name of temporary file */
+	    strlcat(tmpfile, ".UPD", sizeof tmpfile);
 
 	    /* open file1 for input */
 	    f1 = open(file1, O_RDONLY);
 	    if (f1 < 0) {
-		fprintf(stderr, "Unable to open input file %s, ", file1);
-		if (errno >= sys_nerr)
-		    fprintf(stderr, "error code = %d\n", errno);
-		else
-		    fprintf(stderr, "%s\n", sys_errlist[errno]);
+		fprintf(stderr, "Unable to open input file %s: %s\n",
+			file1, strerror(errno));
 		return 1;
 	    }
 
 	    /* open temporary output file */
 	    f2 = open(tmpfile, (O_WRONLY | O_CREAT | O_TRUNC), s1.st_mode);
 	    if (f2 < 0) {
-		fprintf(stderr, "Unable to open output file %s, ", tmpfile);
-		if (errno >= sys_nerr)
-		    fprintf(stderr, "error code = %d\n", errno);
-		else
-		    fprintf(stderr, "%s\n", sys_errlist[errno]);
+		fprintf(stderr, "Unable to open output file %s: %s\n",
+			tmpfile, strerror(errno));
 		fflush(stdout);
 		close(f1);
 		return 1;
@@ -337,8 +330,8 @@ Copy(char *file1, char *file2, short recursive, int level)
 
 	    /* Rename file2 to file2.old. [-r] */
 	    if (renameTargets && goods2) {
-		strcpy(newName, file2);
-		strcat(newName, ".old");
+		strlcpy(newName, file2, sizeof newName);
+		strlcat(newName, ".old", sizeof newName);
 		if (verbose) {
 		    printf("  Renaming %s to %s\n", file2, newName);
 		    fflush(stdout);
@@ -367,7 +360,8 @@ Copy(char *file1, char *file2, short recursive, int level)
 	    if (s1.st_size != s2.st_size) {
 		fprintf(stderr,
 			"WARNING: New file %s is %u bytes long; should be %u\n",
-			file2, s2.st_size, s1.st_size);
+			file2, (unsigned int) s2.st_size,
+			(unsigned int) s1.st_size);
 	    }
 	}
 
@@ -532,8 +526,8 @@ Copy(char *file1, char *file2, short recursive, int level)
 	    return 1;
 	}
 
-	strcpy(f1, file1);
-	strcpy(f2, file2);
+	strlcpy(f1, file1, sizeof f1);
+	strlcpy(f2, file2, sizeof f2);
 	p1 = f1 + strlen(f1);
 	p2 = f2 + strlen(f2);
 	if (p1 == f1 || p1[-1] != '/')
@@ -550,8 +544,8 @@ Copy(char *file1, char *file2, short recursive, int level)
 	while ((d = readdir(dir)) != NULL) {
 	    if (strcmp(d->d_name, ".") == 0 || strcmp(d->d_name, "..") == 0)
 		continue;
-	    strcpy(p1, d->d_name);
-	    strcpy(p2, d->d_name);
+	    strlcpy(p1, d->d_name, sizeof f1 - (p1 - f1));
+	    strlcpy(p2, d->d_name, sizeof f2 - (p2 - f2));
 	    code = Copy(f1, f2, recursive, level + 1);
 	    if (code && !rcode)
 		rcode = 1;	/* remember errors */
@@ -625,7 +619,7 @@ Copy(char *file1, char *file2, short recursive, int level)
 		for (i = 1; i < strlen(file1); i++)
 		    if (file1[i] == '/')
 			break;
-		strcpy(aclspace, &file1[i]);
+		strlcpy(aclspace, &file1[i], sizeof aclspace);
 
 		blob.in_size = 1 + strlen(aclspace);
 		tfd = open(file1, O_RDONLY, 0);
@@ -649,8 +643,8 @@ Copy(char *file1, char *file2, short recursive, int level)
 		/* Now convert the thing. */
 		oacl = (struct OldAcl *)(aclspace + 4);
 		sprintf(tacl, "%d\n%d\n", oacl->nplus, oacl->nminus);
-		strcat(tacl, oacl->data);
-		strcpy(aclspace, tacl);
+		strlcat(tacl, oacl->data, sizeof tacl);
+		strlcpy(aclspace, tacl, sizeof aclspace);
 	    } /*Grab and convert old-style ACL */
 	    else {
 		/* Get a new-style ACL */
@@ -747,7 +741,7 @@ isMountPoint(char *name, struct ViceIoctl *blob)
 	 * No slash appears in the given file name.  Set parent_dir to the current
 	 * directory, and the last component as the given name.
 	 */
-	strcpy(parent_dir, ".");
+	strlcpy(parent_dir, ".", sizeof parent_dir);
 	last_component = true_name;
     }
 
@@ -760,9 +754,9 @@ isMountPoint(char *name, struct ViceIoctl *blob)
 
     blob->in = last_component;
     blob->in_size = strlen(last_component) + 1;
-    blob->out_size = MAXSIZE;
+    blob->out_size = AFS_PIOCTL_MAXSIZE;
     blob->out = space;
-    memset(space, 0, MAXSIZE);
+    memset(space, 0, AFS_PIOCTL_MAXSIZE);
 
     code = pioctl(parent_dir, VIOC_AFS_STAT_MT_PT, blob, 0);
 
@@ -784,5 +778,4 @@ isMountPoint(char *name, struct ViceIoctl *blob)
 	     */
 	}
     }
-    return 4;
 }
